"use strict";

exports.__esModule = true;

var _extends = Object.assign || function (target) { for (var i = 1; i < arguments.length; i++) { var source = arguments[i]; for (var key in source) { if (Object.prototype.hasOwnProperty.call(source, key)) { target[key] = source[key]; } } } return target; };

var _react = require("react");

var useAsync = function useAsync(opts, init) {
  var counter = (0, _react.useRef)(0);
  var isMounted = (0, _react.useRef)(true);
  var lastArgs = (0, _react.useRef)(undefined);
  var abortController = (0, _react.useRef)({ abort: function abort() {} });

  var options = typeof opts === "function" ? { promiseFn: opts, initialValue: init } : opts;
  var promiseFn = options.promiseFn,
      deferFn = options.deferFn,
      initialValue = options.initialValue,
      onResolve = options.onResolve,
      onReject = options.onReject,
      watch = options.watch;

  var _useState = (0, _react.useState)({
    data: initialValue instanceof Error ? undefined : initialValue,
    error: initialValue instanceof Error ? initialValue : undefined,
    startedAt: promiseFn ? new Date() : undefined,
    finishedAt: initialValue ? new Date() : undefined
  }),
      state = _useState[0],
      setState = _useState[1];

  var handleData = function handleData(data) {
    var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {};

    if (isMounted.current) {
      setState(function (state) {
        return _extends({}, state, { data: data, error: undefined, finishedAt: new Date() });
      });
      callback(data);
    }
    return data;
  };

  var handleError = function handleError(error) {
    var callback = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : function () {};

    if (isMounted.current) {
      setState(function (state) {
        return _extends({}, state, { error: error, finishedAt: new Date() });
      });
      callback(error);
    }
    return error;
  };

  var handleResolve = function handleResolve(count) {
    return function (data) {
      return count === counter.current && handleData(data, onResolve);
    };
  };
  var handleReject = function handleReject(count) {
    return function (error) {
      return count === counter.current && handleError(error, onReject);
    };
  };

  var start = function start() {
    if ("AbortController" in window) {
      abortController.current.abort();
      abortController.current = new window.AbortController();
    }
    counter.current++;
    setState(function (state) {
      return _extends({}, state, {
        startedAt: new Date(),
        finishedAt: undefined
      });
    });
  };

  var load = function load() {
    var isPreInitialized = initialValue && counter.current === 0;
    if (promiseFn && !isPreInitialized) {
      start();
      return promiseFn(options, abortController.current).then(handleResolve(counter.current), handleReject(counter.current));
    }
  };

  var run = function run() {
    for (var _len = arguments.length, args = Array(_len), _key = 0; _key < _len; _key++) {
      args[_key] = arguments[_key];
    }

    if (deferFn) {
      lastArgs.current = args;
      start();
      return deferFn.apply(undefined, args.concat([options, abortController.current])).then(handleResolve(counter.current), handleReject(counter.current));
    }
  };

  var cancel = function cancel() {
    counter.current++;
    abortController.current.abort();
    setState(function (state) {
      return _extends({}, state, { startedAt: undefined });
    });
  };

  (0, _react.useEffect)(function () {
    return promiseFn ? load() && undefined : cancel();
  }, [promiseFn, watch]);
  (0, _react.useEffect)(function () {
    return function () {
      return isMounted.current = false;
    };
  }, []);
  (0, _react.useEffect)(function () {
    return abortController.current.abort;
  }, []);

  return (0, _react.useMemo)(function () {
    return _extends({}, state, {
      isLoading: state.startedAt && (!state.finishedAt || state.finishedAt < state.startedAt),
      initialValue: initialValue,
      run: run,
      reload: function reload() {
        return lastArgs.current ? run.apply(undefined, lastArgs.current) : load();
      },
      cancel: cancel,
      setData: handleData,
      setError: handleError
    });
  }, [state]);
};

var unsupported = function unsupported() {
  throw new Error("useAsync requires react@16.7.0-alpha. Upgrade your React version or use the <Async> component instead.");
};

exports.default = _react.useState ? useAsync : unsupported;